% The Scalar BPM Testbench
%close all;
%clear all;

A = 1; % [V/m]
lambda = 1.3e-6; % [m]

% ==== Grid parms ====
%
% Here, the grid is defined in multiples of the wavelength
% This makes sense because the waves are sampled and Fourier transformed. 
% To know how many wavelengths fit into the grid is important to align with the sampling theorem.
%
% The parms ax and ay define the so-called aperture. 
% The aperture is the field of investigation. 
%
% Here, we have a two-dimensional aperture, given by the parms ax and az. 
% The field propagates along the z-axis.
ax = 8*lambda; % [m]
az = 24*lambda; % [m]

% ==== Split-Step propagation ====
%
% The algorithm follows a so-called 'split-step' approach. 
% The system is 'split' and the wave propagates in 'steps' along a pre-defined axis (here z-axis).
% dz is the distance of propagation, which is also defined as a multiple of lambda.
%
% With a dependency to the wavelength, the number of samples along z aligns by definition to the vacuum wavelenth.
% It needs to be considered, that the wavelength will change in a medium with a refractive index n.
% Then, lambda_n = n/lambda.
%
% Here, we have eight samples per wavelength.
dz = lambda/8;

% only a normal number nz is accepted
% because a loop cannot run from 0 to 32,25 .. makes no sense 
% .. so use fix() to make it a natural number :)
nz = fix(az/dz);

% here we define the grid along the z-axis
iz = 1:1:nz;

% and now make the index iz a distance Z by multiplying with the increment dz
Z = iz*dz;

% ==== Spatial frequencies ====
%
% Here's the story about spatial frequencies .. not easy to get at the first read, so play a little with the numbers
% OK, a spatial frequency fx defines or is defined by the angle of propagation (theta)
% 
% fx = sin(theta)/lambda 
%
%        or in terms of 2*pi 
%
% kx = 2*pi*fx
% 
% Vertical incidence means theta=0 which yields fx=kx=0
% At an angle of lets say 30 degrees we get fx=1/(2*lambda) or kx = pi/lambda
% So we can scan the entire range of spatial frequencies and thereby the range of angles of propagation
%
% The great thing is that we can align this definition to the Fourier transformation.
% According to the Fourier theory, the i-th spatial frequency fi = fx(i) = n/X.
% As we divide by the aperture in the x-axis, which is a space dimension, we call the frequency a spatial frequency ... :)
%
% We correlate this definition to our propagating waves as follows
%
% fi = i/X = sin(theta)/lambda
%
%         or in terms of 2*pi
%
% omegai = 2*pi*fi = 2*pi*i/X = 2*pi*sin(theta)/lambda
%
% Given this formula, we can derive every propagation angle from a given spatial frequency
%
% This is the most important to understand for the moment ...

% 16 samples per wavelength ... f_samp = 16/lambda
f_samp = 16/lambda; % [Hz]

% means a space increment of lambda/16 = dx
dx = 1/f_samp; % space increment x

% ... which requires a resolution in the x-aperture of res_x
res_x = nextpow2(fix(ax/dx)); % required resolution x to sample aperture ax

% ... which yields the number of samples in the aperture x, nx = ...
nx = pow2(res_x); % number of samples x

% increments for ..
% .. spatial frequenies: dfx
% .. angle of propagation: dtheta
% .. x-component of the propagation vector: dkx
%
% maximum frequency: fx_max 
% Consider that the number of samples in the aperture nx covers positive and 
% negative angles of propagation. 
% The samples are divided by two:
% positive angles of propagation: (0..nx/2-1) 
% negative angles of propagation: (-nx/2..-1)
dfx = 1/ax;
fx_max = dfx*(nx/2-1);
dtheta = asin(dfx*lambda)*180/pi;
dkx = 2*pi*dfx;

% To align with the frequency ordering of the Matlab Fast Fourier Transformation fft()
% the indice ikx are defined as follows
ikx = [0:1:nx/2-1 -nx/2:1:-1]; % fft2 frequency ordering!

% From a multiplication with the increment, the two following vectors are obtained
KX = dkx*ikx;
FX = dfx*ikx;

% spatial indice follow the same logic and so the aperture is defined by an interval
% [-dx*nx/2 .. +dx*(nx/2-1)]
ix = -nx/2:1:nx/2-1; % spatial index ordering!
X = ix*dx;

% build a twodimensional grid with the matlab function meshgrid()
[XZ,ZX] = meshgrid(X,Z);

% ==== FREQUENCY REVERSE ====
% This is mandatory to obtain the correct signs for the spatial frequencies.
% A spatial frequency fx(i) is multiplied by REV(i)=(-1)^i, i=0 .. n-1
REV = (-1).^([0:nx-1]);

% ==== ANTIALIAS ====
% maximum frequency to avoid aliasing effects
f_max = f_samp/2;
kx_max = 2*pi*f_max;
theta_max = asin(f_max*lambda)*180/pi;

% define a matrix with zeros and ones to eliminate frequencies affected by aliasing
% a multiplication with the dot-operator (ALI.*FFT_OF_FIELD) will remove the related
% frequency components 
ALI=ones(1,nx);
ALI(find(abs(FX)>=f_max))=0;

% Just some matrices for system definitions ..
% a system is defined by its distribution of the complex refractive index
% n(x)=nr(x)-i*k(x)
N = zeros(nz,nx);
N2 = zeros(nz,nx);
N3 = zeros(nz,nx);

% ==== Systems ====
% some systems to play with ..
% just set the variable system to one of the 
% alternatives in the case statement
system = 'lens';

switch system
    
    case 'homog'
        % ==== homogeneous system ====
        n0 = 1.6;
        N = n0 * ones(nz,nx);
        
    case 'boundary'
        % ==== single boundary ====
        n1 = 1;
        n2 = 1.15;
        N = [n1*ones(nz/2,nx);n2*ones(nz/2,nx)];
        
    case 'waveguide'        
        % ==== Waveguide ===
        n_core = 1.8;
        n_clad = 1;
        D = 5*lambda; % [m]
        
        index_x0 = fix(nx/2);
        
        for index_z = 1:nz
            for index_x = 1:nx
                
                if D/2 >= abs((index_x - index_x0)*dx)
                    N(index_z,index_x) = n_core;
                else
                    N(index_z,index_x) = n_clad;
                end
                
            end
        end
        
    case 'lens'
        % ==== Lens ====
        
        % TEST TEST TEST
        Rl = 7*lambda; % radius left
        Rr = -7*lambda; % radius right

        % bi-convex
        %Rl = 10*lambda; % radius left
        %Rr = -5*lambda; % radius right
        
        % bi-convex
        %Rl = 5*lambda; % radius left
        %Rr = -10*lambda; % radius right
        
        % bi-concave
        %Rl = -10*lambda; % radius left
        %Rr = 5*lambda; % radius right
        
        % bi-concave
        %Rl = -5*lambda; % radius left
        %Rr = 10*lambda; % radius right
        
        % concave-convex
        %Rl = 5*lambda; % radius left
        %Rr = 10*lambda; % radius right
        
        % convex-concave
        %Rl = -5*lambda; % radius left
        %Rr = -10*lambda; % radius right
        
        D = ax; % lens diameter
        
        Tl = Rl - Rl * sqrt( 1 - 1/4*(D/Rl)^2 ) % thickness left hemisphere
        Tr = Rr - Rr * sqrt( 1 - 1/4*(D/Rr)^2 ) % thickness right hemisphere
        Td = 1*lambda; % lens thickness, for thin lenses set to zero
        
        % position of the lens
        index_x0 = fix(nx/2);
        index_z0 = fix(nz/2);
        
        n_lens = 1.8;
        n_medium = 1;
        
        Tl_inum = fix(abs(Tl)/dz);
        Tr_inum = fix(abs(Tr)/dz);
        Td_inum = fix(abs(Td/2)/dz);
        T_inum = Tl_inum + Tr_inum + 2*Td_inum;
        
        Rl_inum = fix(abs(Rl)/dz);
        Rr_inum = fix(abs(Rr)/dz);
        
        left_center = index_z0 - Tl_inum + Rl_inum - Td_inum;
        right_center = index_z0 + Tr_inum - Rr_inum + Td_inum;
        
        for index_z = 1:nz
            for index_x = 1:nx
               
                this_n = n_medium;
                
                %         if index_z>=index_z0 % right sphere
                %             if Rr^2 >= ( ((index_x-index_x0)*dx)^2 + ((index_z-index_z0)*dz)^2 )
                %                 this_n = n_lens;
                %             end
                %         else % left sphere
                %             disp(' left ');
                %             if Rl^2 >= ( ((index_x-index_x0)*dx)^2 + ((index_z0-index_z)*dz)^2 )
                %                 this_n = n_lens;
                %             end
                %         end
                
                this_index_z = index_z;
                
                % left hemisphere of lens
                if ((max(index_z0 - Tl_inum - Td_inum, 1) <= index_z) && (index_z <= index_z0 ))
                    %disp(sprintf('Left hemisphere: (%d,%d)',index_x*dx,index_z*dz));
                    if Rl>0
                        if (Rl^2 >= ( ((index_x-index_x0)*dx)^2 + ((left_center-index_z)*dz)^2 ))
                            this_n = n_lens;
                        else
                            this_n = n_medium;
                        end;
                        this_index_z = index_z;
                    else
                        if (Rl^2 >= ( ((index_x-index_x0)*dx)^2 + ((left_center-index_z)*dz)^2 ))
                            this_n = n_medium;
                        else
                            this_n = n_lens;
                        end
                        this_index_z = index_z0 - ( index_z - ( index_z0 - Tl_inum - Td_inum ) ) - 1;
                    end
                end
                
                % right hemisphere of lens
                if ((index_z0 <= index_z) && (index_z <= min(index_z0 + Tr_inum + Td_inum, nz)))
                    %disp(sprintf('Right hemisphere: (%d,%d)',index_x*dx,index_z*dz));
%                     if (Rr^2 >= ( ((index_x-index_x0)*dx)^2 + ((index_z-right_center)*dz)^2 ))
%                         this_n = n_lens;
%                     end
                    if Rr<0
                        if (Rr^2 >= ( ((index_x-index_x0)*dx)^2 + ((index_z-right_center)*dz)^2 ))
                            this_n = n_lens;
                        else
                            this_n = n_medium;
                        end
                        this_index_z = index_z;
                    else
                        if (Rr^2 >= ( ((index_x-index_x0)*dx)^2 + ((index_z-right_center)*dz)^2 ))
                            this_n = n_medium;
                        else
                            this_n = n_lens;
                        end
                        this_index_z = index_z0 + ( Tr_inum + Td_inum - ( index_z - index_z0 ) );
                    end
                end
                                
                N(this_index_z,index_x) = this_n;
                
                %        if this_n~=n_medium
                %            disp(sprintf('E(%d,%d) = %d',index_x,index_z,this_n));
                %        end
                
            end
        end
        
        %============================
        % Equivalent GRIN lens
        %============================
        
        % lens thickness in aperture ax for adjusted lens thickness
        theta_max_left = asin( ax/(2*abs(Rl)) );
        l_adjust = abs(Rl) * cos(theta_max_left); 
        theta_max_right = asin( ax/(2*abs(Rr)) );
        r_adjust = abs(Rr) * cos(theta_max_right);
        
        % length of the rod

        % BAD NEWS FOR THE REFRACTIVE INDEX AT NX/2
        %d_rod = abs(Tr) + abs(Tl) + abs(Td);
        
        % GOOD NEWS FOR THE REFRACTIVE INDEX AT NX/2
        %d_rod = abs(Td);
        
        % GOOD NEWS FOR THE REFRACTIVE INDEX AT NX/2
        d_rod = 0.5*lambda;
        d_rod_inum = fix(abs(d_rod)/dz);
        
        for index_z = 1:nz
            for index_x = 1:nx
                
                if ( abs((index_z0-index_z))*dz ) <= ( d_rod/2 )
 
                    % total lens thickness in infinite aperture
                    dl_tot = abs(Rl) * sqrt( 1 - ( (index_x-index_x0)*dx/abs(Rl) )^2 );
                    dr_tot = abs(Rr) * sqrt( 1 - ( (index_x-index_x0)*dx/abs(Rr) )^2 );
                    
                    % adjusted lens thickness in limited aperture ax
                    dl = max(0,dl_tot - l_adjust);
                    dr = max(0,dr_tot - r_adjust);
                    
                    if Rl<0 % concave
                        dl = Tl - dl;
                    end
                    
                    if Rr>0 % concave
                        dr = Tr - dr;
                    end
                                                         
                    % refractive index of GRIN rod
                    % obtained from optical path length
                    % n_rod(x) * d_rod = n_lens * d_lens(x)
                    d_rel = ( dl + dr + Td ) / d_rod;
                    n_rod = d_rel * n_lens;
                    
                    N2(index_z,index_x) = n_rod/d_rod_inum;
                    
                    if index_x==1 || index_x==nx/2 || index_x==nx
                        
                        disp(sprintf('CHECK Optical Path Length: ix = %d, iz = %d *** OPL(lens) = %.5d, OPL(rod) = %.5d, DELTA = %.10d', ...
                            index_x, ...
                            index_z, ...
                            (abs(dl)+abs(dr)+abs(Td))*n_lens, ...
                            d_rod*n_rod, ...
                            abs( (abs(dl)+abs(dr)+abs(Td))*n_lens - d_rod*n_rod) ) );
                        
                    end
                    
                else
                    
                    N2(index_z,index_x) = n_medium;
                
                end
                
            end
        end
        
        %============================
        % equivalent KERR media
	% TODO: under construction .. 
        %============================
        
        d_kerr = 4*lambda;
        for index_z = 1:nz
            for index_x = 1:nx
                
                if ( abs((index_z0-index_z))*dz ) <= ( d_kerr/2 )
                    
                    N3(index_z,index_x) = -1; % indicates KERR medium
                    
                else
                    
                    N3(index_z,index_x) = n_medium;
                    
                end
                
            end
        end
        
    case 'slit'
        
        index_x0 = fix(nx/2);
        index_z0 = fix(2e-6/dz);
        
        n_medium = 1+1i*0;
        n_slit = 1.4+1i*1;
        
        T = 0.25*lambda;
        D = 4*lambda;
        
        T_inum = fix(abs(T)/dz);
        
        for index_z = 1:nz
            for index_x = 1:nx
                
                N(index_z,index_x) = n_medium;
                
                if T/2 >= abs((index_z - index_z0)*dz)
                    if D/2 <= abs((index_x - index_x0)*dx)
                        N(index_z,index_x) = n_slit;
                    end
                end
                
            end
        end
        
    otherwise
        disp('Unknown system ...');
        
end

% ==== Input ====
% some inputs to play with ..
% angle of incidence in degrees (0 to plus/minus 90): theta_deg
% frequency of the input field: f_sig
theta_deg = 0;
f_sig = 0;

test = 7;

switch test
    
    case 0
        % === TEST 0 ====
        f_sig = 0; %[Hz]
        theta_deg = 0; % [deg]

    case 1
        % ==== TEST 1.1, 5 degrees ====
       theta_deg = 5; % [deg]
       f_sig = 1/lambda*sin(theta_deg*pi/180)/1e6; % [MHz]

    case 2
        % ==== TEST 1.2, 100 kHz (5 degrees) ====
        f_sig = 0.1; %[MHz]

        % lambda = 1.3e-6;
        % ax = 20*lambda;
        % az = 20*lambda;
        % dz = lambda/8;
        % f_samp = 4/lambda;
        %f_sig = FX(3)/1e6; %[MHz]
        
        theta_deg = asin(f_sig*1e6*lambda)*180/pi; % [deg]

    case 3        
        % === TEST 2.1, 30 degrees ====
       theta_deg = 30; % [deg]
       f_sig = 1/lambda*sin(theta_deg*pi/180)/1e6; % [MHz]
        
    case 4
        % ==== TEST 2.2, 400 kHz (31.33 degrees)====
        f_sig = 0.400; % [MHz]
        
        % lambda = 1.3e-6;
        % ax = 20*lambda;
        % az = 20*lambda;
        % dz = lambda/8;
        % f_samp = 4/lambda;
        %f_sig = FX(11)/1e6; %[MHz]
        
        theta_deg = asin(f_sig*1e6*lambda)*180/pi; % [deg]
        
    case 5
        % ==== TEST 3.1, 45 degrees ====
        theta_deg = 45; % [deg]
        f_sig = 1/lambda*sin(theta_deg*pi/180)/1e6; % [MHz]
        
    case 6
        % ==== TEST 3.2, 550 kHz (45 degrees) ====
        f_sig = 0.55; %[MHz]
        theta_deg = asin(f_sig*1e6*lambda)*180/pi; % [deg]
    
    case 7
        % ==== TEST i, theta(i) degrees ====
        f_sig = FX(1)/1e6; %[MHz]
        theta_deg = asin(f_sig*1e6*lambda)*180/pi; % [deg]
        
        %theta_deg = 10;
        theta_deg = 0;
        
    otherwise
        disp('Unknown test ...');
        exit;
end

theta_rad = theta_deg * pi/180; % [rad]
k0 = 2*pi/lambda;
kx = k0*sin(theta_rad);
kz = k0*cos(theta_rad);

% === CHECK ===
disp(sprintf('CHECK angle (atan): theta = %d [deg]',atan(kx/kz)*180/pi));
disp(sprintf('CHECK angle (asin): theta = %d [deg]',asin(kx/2/pi*lambda)*180/pi));
disp(sprintf('CHECK frequency: f = %.2d [MHz]',kx/2/pi/1e6));

E = zeros(nz,nx);
E2 = zeros(nz,nx);
E3 = zeros(nz,nx);

% ==== Input ====
% some inputs to play with ..
% Gaussian shape: wave = 'gauss'
% Plane wave: wave = 'plane'

wave = 'gauss';

switch(wave)

    case 'plane'
        
        E(1,:) = A*exp(1i* ( kx*XZ(1,:) + kz*ZX(1,:) ) );
        E2(1,:) = A*exp(1i* ( kx*XZ(1,:) + kz*ZX(1,:) ) );
        E3(1,:) = 1E40*A*exp(1i* ( kx*XZ(1,:) + kz*ZX(1,:) ) );
                
    case 'gauss'
        
        %         zR = pi * (W0*W0) / lambda; % Raleigh range [m]
        %         Wz = zR * ( zR + ( z0*z0 / zR ) ); % beam width at z [m]
        %         Rz = z0 + ( zR*zR / z0 ); % radius curvature of the wavefront at z [m]
        %
        %         % ==== GAUSSIAN ====
        %         E = round(( E0/zR * W0/Wz ) * ( exp(-((XY).*(XY)+(YX).*(YX))/(Wz*Wz)) .* exp(-1i* (k*z0 -atan(z0/zR) +k*((XY).*(XY)+(YX).*(YX))/(2*Rz)) ) ),digits);
        
        % ==== GAUSSIAN with z0=0 ====
        W0 = 3*lambda; % [m]
        E(1,:) = A*exp(-XZ(1,:).^2/W0^2).*exp( 1i * ( kx*(XZ(1,:)) + kz*ZX(1,:) ) );
        E2(1,:) = A*exp(-XZ(1,:).^2/W0^2).*exp( 1i * ( kx*(XZ(1,:)) + kz*ZX(1,:) ) );
        E3(1,:) = 1E2*A*exp(-XZ(1,:).^2/W0^2).*exp( 1i * ( kx*(XZ(1,:)) + kz*ZX(1,:) ) );
                
    otherwise
        sprintf('Unknown wave ...')
        
end

% PLOT ...
S = fft2(E(1,:));
index = find(abs(S)~=0);
index_fmax = find(FX(index)==max(FX(index)));
index_fmin = find(FX(index)==min(FX(index)));
disp(sprintf('Maximum frequency in input signal: %.3d [MHz] (Theta = %.2d)',FX(index_fmax)/1e6,asin(FX(index_fmax)*lambda)*180/pi));
disp(sprintf('Minimum frequency in input signal: %.3d [MHz] (Theta = %.2d)',FX(index_fmin)/1e6,asin(FX(index_fmin)*lambda)*180/pi));

% figure(100);
% F = KX/2/pi/1e6; % [MHz]
% stem(F,abs(S));grid on;title(sprintf('Spectrum of plane wave \\newline with f_{sig} = %.3d [MHz] (\\Theta = %d [deg])',f_sig,asin(f_sig/1e-6*lambda)*180/pi));xlim([min(F) max(F)]);ylim([min(abs(S)) max(abs(S))]);
% xlabel('f [MHz]');
% ylabel('E [V/m]');

% ==== Propagation Method ====
% Here we go ...
% .. swpm = 'Scalar Wave Propagation Method'
% .. sbpm = 'Scalar Beam Propagation Method'
% 
% We want to work with the Wave Propagation Method because it is 
% not limited to paraxial propagation, i.e. +/- 5 degrees
%
% Scalar Wave Propagation Method: method = 'swpm'
% Scalar Beam Propagation Method: method = 'sbpm'

method = 'swpm';

switch(method)
    
    case 'swpm'
        
        % Simulate N (Spheric lens)
        for index_z=1:nz-1
            
            % ==== SPECTRUM ====
            S = fft2( E(index_z,:) );
            
            for index_kx = 1:nx
                
                for index_x = 1:nx
                    
                    % ==== PROPAGATION VECTOR ====
                    nk0 = real( N(index_z,index_x) )*k0;
                    kk0 = imag( N(index_z,index_x) )*k0;
                    KZ = ( nk0 * sqrt( 1 - (KX(index_kx)/nk0)^2) );
                    
                    % ==== FRESNEL COEFFICIENTS ====
		    % Exercise:
		    % The Fresnel coefficients can be expressed in terms of (kx,kz) in the two-dimensional
		    % and (kx,ky,kz) in the three-dimensional case.
		    % Read the paper which I sent you with this file and implement the formula with Matlab
		    % (the code does not fail with F=1, so you can first play a little with the parameters)
		    % every entry of F is a function of space and frequency: F(x,z,kx,kz)
                    F = 1; % TODO
                    
                    if( isreal( KZ ) ) % exclude evanescent waves
                        
                        % ==== Superposition of plane waves ====
                        E(index_z+1,index_x) = E(index_z+1,index_x) + ( F * (-1)^(index_kx-1) * S(index_kx)/nx * exp(-kk0*dz) * exp( 1i* ( XZ(index_z,index_x)*KX(index_kx) + KZ*dz ) ) );
                        
                    end
                    
                end
                
            end
            
        end
            
        % Simulate N2 (GRIN lens)
        for index_z=1:nz-1
            
            % ==== SPECTRUM ====
            S = fft2( E2(index_z,:) );
            
            for index_kx = 1:nx
                
                for index_x = 1:nx
                    
                    % ==== PROPAGATION VECTOR ====
                    nk0 = real( N2(index_z,index_x) )*k0;
                    kk0 = imag( N2(index_z,index_x) )*k0;
                    KZ = ( nk0 * sqrt( 1 - (KX(index_kx)/nk0)^2) );
                    
                    % ==== FRESNEL COEFFICIENTS ====
                    F = 1; % TODO
                    
                    if( isreal( KZ ) ) % exclude evanescent waves
                        
                        % ==== Superposition of plane waves ====
                        E2(index_z+1,index_x) = E2(index_z+1,index_x) + ( F * (-1)^(index_kx-1) * S(index_kx)/nx * exp(-kk0*dz) * exp( 1i* ( XZ(index_z,index_x)*KX(index_kx) + KZ*dz ) ) );
                        
                    end
                    
                end
                
            end
            
        end

        % Simulate N3 (Kerr Medium)
        for index_z=1:nz-1
            
            % ==== SPECTRUM ====
            S = fft2( E3(index_z,:) );
            
            for index_kx = 1:nx
                
                for index_x = 1:nx
                    
                    % ==== PROPAGATION VECTOR ====
                    % K = 4.4E-12; % Kerr constant [m/V^2] for Nitrobenzene
                    % S2 = lambda * K;
                    % n2 = ;
                    n0 = n_lens; % glas
                    n2 = 10E-20; % second ortder nonlinear refractive index for glass [m^2/W]
                    % n2 = 1E31; % FAKE second order nonlinear refractive index for glass [m^2/W]
                    if -1==N3(index_z,index_x) % Kerr medium                        
                       % this_n = n0 + S2*(S(index_kx)*S(index_kx)'/pow2(nx)); % S2-term causes nonlinear electric DC Kerr effect
                       this_n = n0 + n2 * (S(index_kx)*S(index_kx)'/pow2(nx)); % S2-term causes nonlinear electric AC Kerr effect 
                       N3(index_z,index_x) = this_n;
                       disp(sprintf('Kerr refractive index(ix=%d,iz=%d): %.8d, Intensity: %.8d',index_x,index_z,this_n,S(index_kx)*S(index_kx)'/pow2(nx)));
                    else
                        this_n = N3(index_z,index_x);                        
                    end
                    
                    nk0 = real( this_n )*k0;
                    kk0 = imag( this_n )*k0;
                    KZ = ( nk0 * sqrt( 1 - (KX(index_kx)/nk0)^2) );
                    
                    % ==== FRESNEL COEFFICIENTS ====
                    F = 1; % TODO
                    
                    if( isreal( KZ ) ) % exclude evanescent waves
                        
                        % ==== Superposition of plane waves ====
                        E3(index_z+1,index_x) = E3(index_z+1,index_x) + ( F * (-1)^(index_kx-1) * S(index_kx)/nx * exp(-kk0*dz) * exp( 1i* ( XZ(index_z,index_x)*KX(index_kx) + KZ*dz ) ) );
                        
                    end
                    
                end
                
            end
            
        end
        
    case 'sbpm'
        
        for j=1:nz-1
            
            %      F_SHIFT = fft2(E(j,:));
            %      E(j+1,:) = ifft2(F_SHIFT);
            
            % ==== PROPAGATION VECTOR ====
            na = sum(real(N(j,:)))/nx;
            nk0 = na*k0;
            KZ = (nk0 * sqrt(1-(KX/nk0).^2));
                        
            % ==== FRESNEL COEFFICIENTS ====
	    % Exercise:
	    % The Fresnel coefficients can be expressed in terms of (kx,kz) in the two-dimensional
	    % and (kx,ky,kz) in the three-dimensional case.
	    % Read the paper which I sent you with this file and implement the formula with Matlab
	    % (the code does not fail with F=1, so you can first play a little with the parameters)
	    % every entry of F is a function of frequency and the average refracive index before (n_ave_i) and
	    % behind (n_ave_t) the boundary: F(n_ave_i, n_ave_t, kx, ky)
            F = 1; % TODO
            
            % ==== SUPRESS EVANESCENT FREQs ====
	    % Evanescent frequencies occur from total internal reflection
	    % and are identified by an imaginary kz=sqrt(k^2-kx^2)
            EV = ones(1,nx);
            EV(find(0~=imag(KZ))) = 0;
            
	    % derive the propagated spatial frequency
	    % propagation over a distance dz
	    % suppress evanescent waves (EV)
	    % suppress aliasing waves (ALI)
	    % ... for the moment :)
            F_SHIFT = EV.*ALI.*(F.*fft2(E(j,:))).*exp(1i*KZ*dz);

	    % Inverse FFT yields field distibution
	    % Field at (j+1)*dz is obtained from field at j*dz by propagating a distance
	    % (j+1)*dz - j*dz = dz
            E(j+1,:) = ifft2(F_SHIFT).*exp(1i*k0*( real(N(j,:))-na )*dz).*exp(-k0*imag(N(j,:))*dz);
            
%             EV.*atan(KX./KZ)*180/pi            
%             pause;
            
        end
        
    otherwise
        disp('Unknown method ...');
        
end
    
figure(200);hold off;
subplot(331);surf(ZX,XZ,real(N),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Refractive index');daspect([1 1 1]);
subplot(332);surf(ZX,XZ,real(E),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Electric field');daspect([1 1 1]);
subplot(333);contour(ZX,XZ,real(E),5);view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Electric field');daspect([1 1 1]);
subplot(334);surf(ZX,XZ,real(N2),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Refractive index');daspect([1 1 1]);
subplot(335);surf(ZX,XZ,real(E2),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Electric field');daspect([1 1 1]);
subplot(336);contour(ZX,XZ,real(E2),5);view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Electric field');daspect([1 1 1]);
subplot(337);surf(ZX,XZ,real(N3),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Refractive index');daspect([1 1 1]);
subplot(338);surf(ZX,XZ,real(E3),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Electric field');daspect([1 1 1]);
subplot(339);contour(ZX,XZ,real(E3),5);view(2);ylim([min(X) max(X)]);xlim([min(Z) max(Z)]);title('Electric field');daspect([1 1 1]);

figure(300);hold on;grid on;
FZ = real(sqrt((1/lambda)^2-FX.^2));
stem(ikx*dtheta,FX/1e6);
stem(ikx*dtheta,FZ/1e6);
stem(90,min(FX/1e6),'k*');
stem(90,max(FX/1e6),'k*');
stem(-90,min(FX/1e6),'k*');
stem(-90,max(FX/1e6),'k*');
stem(180,min(FX/1e6),'g*');
stem(180,max(FX/1e6),'g*');
stem(-180,min(FX/1e6),'g*');
stem(-180,max(FX/1e6),'g*');
title('Spatial frequencies over index');
xlabel('\Theta [deg]');
ylabel('Spatial frequency [MHz]');
xlim([min(ikx*dtheta) max(ikx*dtheta)]);
ylim([min(FX/1e6) max(FX/1e6)]);
legend('f_x(ik_x)','f_z(ik_x)','90 degrees','location','northwest');

figure(500);
S = fft2(E(1,:));
subplot(321);
stem(FX/1e6,abs(S));grid on;title(sprintf('Spectrum of the incident wave \\newline with f_{\\Theta} = %.3d [MHz] (\\Theta = %d [deg])',f_sig,asin(f_sig*1e6*lambda)*180/pi));
xlim([min(FX/1e6) max(FX/1e6)]);ylim([min(abs(S)) max(abs(S))]);
xlabel('f [MHz]');ylabel('E [V/m]');
% subplot(132);
% surf(ZX,XZ,real(E),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);title('Electric field');daspect([1 1 1]);
% grid off;
% xlabel('z [m]'); ylabel('real(E)~[V/m]');
% %xlim([0 az]);ylim([-ax/2 ax/2]);
% xlim([min(Z) max(Z)]);ylim([min(X) max(X)]);
S = fft2(E(nz-1,:));
subplot(322);
stem(FX/1e6,abs(S));grid on;title(sprintf('Spectrum of the \\newline propagated wave'));
xlim([min(FX/1e6) max(FX/1e6)]);ylim([min(abs(S)) max(abs(S))]);
xlabel('f [MHz]');ylabel('E [V/m]');

S = fft2(E2(1,:));
subplot(323);
stem(FX/1e6,abs(S));grid on;title(sprintf('Spectrum of the incident wave \\newline with f_{\\Theta} = %.3d [MHz] (\\Theta = %d [deg])',f_sig,asin(f_sig*1e6*lambda)*180/pi));
xlim([min(FX/1e6) max(FX/1e6)]);ylim([min(abs(S)) max(abs(S))]);
xlabel('f [MHz]');ylabel('E [V/m]');
% subplot(132);
% surf(ZX,XZ,real(E),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);title('Electric field');daspect([1 1 1]);
% grid off;
% xlabel('z [m]'); ylabel('real(E)~[V/m]');
% %xlim([0 az]);ylim([-ax/2 ax/2]);
% xlim([min(Z) max(Z)]);ylim([min(X) max(X)]);
S = fft2(E2(nz-1,:));
subplot(324);
stem(FX/1e6,abs(S));grid on;title(sprintf('Spectrum of the \\newline propagated wave'));
xlim([min(FX/1e6) max(FX/1e6)]);ylim([min(abs(S)) max(abs(S))]);
xlabel('f [MHz]');ylabel('E [V/m]');

S = fft2(E3(1,:));
subplot(325);
stem(FX/1e6,abs(S));grid on;title(sprintf('Spectrum of the incident wave \\newline with f_{\\Theta} = %.3d [MHz] (\\Theta = %d [deg])',f_sig,asin(f_sig*1e6*lambda)*180/pi));
xlim([min(FX/1e6) max(FX/1e6)]);ylim([min(abs(S)) max(abs(S))]);
xlabel('f [MHz]');ylabel('E [V/m]');
% subplot(132);
% surf(ZX,XZ,real(E),'FaceColor','interp','FaceLighting','flat','LineStyle','none');view(2);title('Electric field');daspect([1 1 1]);
% grid off;
% xlabel('z [m]'); ylabel('real(E)~[V/m]');
% %xlim([0 az]);ylim([-ax/2 ax/2]);
% xlim([min(Z) max(Z)]);ylim([min(X) max(X)]);
S = fft2(E3(nz-1,:));
subplot(326);
stem(FX/1e6,abs(S));grid on;title(sprintf('Spectrum of the \\newline propagated wave'));
xlim([min(FX/1e6) max(FX/1e6)]);ylim([min(abs(S)) max(abs(S))]);
xlabel('f [MHz]');ylabel('E [V/m]');



disp(sprintf('Method: %s',method));
disp(sprintf('X = %.2d, Z = %.2d, nx = %d, nz = %d, dx = %.2d, dz = %.2d, dkx = %.2d [2\\pi MHz], dfx = %.2d [MHz], f_samp = %.2d [MHz]',ax,az,nx,nz,dx,dz,dkx/1e6,dfx/1e6,f_samp/1e6));
disp(sprintf('%s wave, lambda = %d, theta = %d, sin(%d*pi/180) = %d',wave,lambda,theta_deg,theta_deg,sin(theta_rad)));
disp(sprintf('Sampling frequency: %.2d [Hz]',f_samp));
disp(sprintf('Signal frequency: %.2d [Hz]',f_sig*1e6));
disp(sprintf('Num of filtered frequencies: %d [-]',size(find(ALI==0),2)));
disp(sprintf('Filtered: %d [MHz]\n',find(ALI==0)*dkx/1e6));

disp('Done.')

